import { Container, DefaultProperties } from '@react-three/uikit';
import { colors } from './theme';
import React, { createContext, memo, useCallback, useContext, useEffect, useMemo, useRef, useState, } from 'react';
import { X } from '@react-three/uikit-lucide';
const DialogAnchorContext = createContext(undefined);
export function DialogAnchor({ children }) {
    const [element, setElement] = useState(undefined);
    const set = useCallback((prevElement, element) => setElement((e) => (e === prevElement ? element : e)), []);
    return (React.createElement(React.Fragment, null,
        React.createElement(DialogAnchorProvider, { set: set, children: children }),
        element));
}
const DialogAnchorProvider = memo(({ children, set }) => {
    return React.createElement(DialogAnchorContext.Provider, { value: set }, children);
});
const DialogContext = createContext(undefined);
export function useDialogContext() {
    const ctx = useContext(DialogContext);
    if (ctx == null) {
        throw new Error(`Can only be used inside a <Dialog> component.`);
    }
    return ctx;
}
export function Dialog({ children, open: providedOpen, onOpenChange, defaultOpen, }) {
    const [uncontrolled, setUncontrolled] = useState(defaultOpen ?? false);
    const open = providedOpen ?? uncontrolled;
    const setElement = useContext(DialogAnchorContext);
    if (setElement == null) {
        throw new Error(`Can only be used inside a <DialogAnchor> component.`);
    }
    const contentRef = useRef(undefined);
    const displayedRef = useRef(undefined);
    useEffect(() => {
        if (!open) {
            setElement(displayedRef.current, undefined);
            displayedRef.current = undefined;
            return;
        }
        if (contentRef.current == null) {
            return;
        }
        setElement(undefined, contentRef.current);
        displayedRef.current = contentRef.current;
    }, [open, setElement]);
    const ref = useRef(onOpenChange);
    ref.current = onOpenChange;
    const openWasProvided = providedOpen != null;
    const value = useMemo(() => ({
        setContent(content) {
            if (displayedRef.current != null) {
                setElement(displayedRef.current, content);
                displayedRef.current = content;
            }
            contentRef.current = content;
        },
        setOpen(open) {
            if (!openWasProvided) {
                setUncontrolled(open);
            }
            ref.current?.(open);
        },
    }), [openWasProvided, setElement]);
    return React.createElement(DialogContext.Provider, { value: value }, children);
}
export function DialogTrigger({ children }) {
    const { setOpen } = useDialogContext();
    return React.createElement(Container, { onClick: () => setOpen(true) }, children);
}
export function DialogOverlay(props) {
    return (React.createElement(Container, { onPointerMove: (e) => e.stopPropagation(), onPointerEnter: (e) => e.stopPropagation(), onPointerLeave: (e) => e.stopPropagation(), onWheel: (e) => e.stopPropagation(), positionType: "absolute", inset: 0, zIndexOffset: 50, backgroundColor: "black", backgroundOpacity: 0.8, ...props }));
}
export function useCloseDialog() {
    const { setOpen } = useDialogContext();
    return useCallback(() => setOpen(false), [setOpen]);
}
export function DialogContentPrimitive({ children }) {
    const dialogContext = useDialogContext();
    useEffect(() => dialogContext.setContent(React.createElement(DialogContext.Provider, { value: dialogContext }, children)));
    return null;
}
export function DialogContent({ children, sm, ...props }) {
    const close = useCloseDialog();
    return (React.createElement(DialogContentPrimitive, null,
        React.createElement(DialogOverlay, { onClick: (e) => {
                close();
                e.stopPropagation();
            }, alignItems: "center", justifyContent: "center" },
            React.createElement(Container, { onClick: (e) => e.stopPropagation(), positionType: "relative", flexDirection: "column", width: "100%", gap: 16, border: 1, backgroundColor: colors.background, padding: 24, sm: { borderRadius: 8, ...sm }, ...props },
                children,
                React.createElement(X, { color: colors.mutedForeground, onClick: close, cursor: "pointer", positionType: "absolute", zIndexOffset: 50, positionRight: 16, positionTop: 16, borderRadius: 2, opacity: 0.7, backgroundOpacity: 0.7, hover: { opacity: 1, backgroundOpacity: 1 }, width: 16, height: 16 })))));
}
export function DialogHeader({ children, ...props }) {
    return (React.createElement(Container, { flexDirection: "column", gap: 6, ...props },
        React.createElement(DefaultProperties, { horizontalAlign: "center", sm: { horizontalAlign: 'left' } }, children)));
}
export function DialogFooter(props) {
    return (React.createElement(Container, { flexDirection: "column-reverse", sm: { flexDirection: 'row', justifyContent: 'flex-end' }, gap: 8, ...props }));
}
export function DialogTitle({ children }) {
    return (React.createElement(DefaultProperties, { fontSize: 18, lineHeight: 1, letterSpacing: -0.4, fontWeight: "semi-bold" }, children));
}
export function DialogDescription({ children }) {
    return (React.createElement(DefaultProperties, { fontSize: 14, lineHeight: 1.43, color: colors.mutedForeground }, children));
}
